@@ -4,7 +4,7 @@ use Pcore -class;
 use Pcore::AE::Handle::Cache::Storage;
 use Pcore::Util::Scalar qw[refaddr];
 
-has default_timeout => ( is => 'ro', isa => PositiveInt, default => 4 );
+has default_keepalive_timeout => ( is => 'ro', isa => PositiveInt, default => 4 );
 
 has handle     => ( is => 'ro', isa => HashRef, default => sub { {} }, init_arg => undef );
 has connection => ( is => 'ro', isa => HashRef, default => sub { {} }, init_arg => undef );
@@ -17,7 +17,7 @@ sub clear ($self) {
     return;
 }
 
-sub store ( $self, $h, $timeout = undef ) {
+sub store ( $self, $h, $keepalive_timeout = undef ) {
 
     # do not cache destroyed handles
     return if $h->destroyed;
@@ -61,12 +61,14 @@ sub store ( $self, $h, $timeout = undef ) {
     $h->on_eof($destroy);
     $h->on_read($destroy);
     $h->on_timeout(undef);
+
     $h->timeout_reset;
-    $h->timeout( $timeout || $self->default_timeout );
+    $h->timeout( $keepalive_timeout || $self->default_keepalive_timeout );
 
     return;
 }
 
+# TODO detect, if $h is not closed
 sub fetch ( $self, $key ) {
     return if !exists $self->{connection}->{$key};
 
@@ -84,6 +86,8 @@ sub fetch ( $self, $key ) {
         }
     }
 
+    # return if $h->destroyed;
+
     $h->on_error(undef);
     $h->on_eof(undef);
     $h->on_read(undef);
@@ -94,16 +98,6 @@ sub fetch ( $self, $key ) {
 }
 
 1;
-## -----SOURCE FILTER LOG BEGIN-----
-##
-## PerlCritic profile "pcore-script" policy violations:
-## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
-## | Sev. | Lines                | Policy                                                                                                         |
-## |======+======================+================================================================================================================|
-## |    3 | 13, 15               | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
-## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
-##
-## -----SOURCE FILTER LOG END-----
 __END__
 =pod
 
